/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzQueue_h
#include "ZzQueue.h"
#endif //ZzQueue_h

#import <mach/mach.h>
#import <mach/mach_error.h>
#import <mach/mach_time.h>

ZzQueue::ZzQueue(const ZzQueue&)
{
	ZzAssert(0); //it's private so don't worry.
}

ZzQueue::ZzQueue(int log2MaxItems)
{
	fLog2MaxItems = log2MaxItems;
	fItems = new ZzQueueItem[1 << fLog2MaxItems];
    kern_return_t kerr;
	kerr = semaphore_create(mach_task_self(), &fSemaphore, SYNC_POLICY_FIFO, 0);
    ZzAssert(kerr == 0);
	fWritePos = 0;
	fReadPos = 0;
}

ZzQueue::~ZzQueue()
{
	semaphore_destroy(mach_task_self(), fSemaphore);
	delete [] fItems;
}

void ZzQueue::Notify(void* aa, void* bb, void* cc, void* dd)
{
	fItems[WriteIndex()].aa = aa; 		
	fItems[WriteIndex()].bb = bb; 		
	fItems[WriteIndex()].cc = cc; 		
	fItems[WriteIndex()].dd = dd;
	fWritePos++;
	SignalSemaphore();	
}

void ZzQueue::Wait(void** aa, void** bb, void** cc, void** dd)
{
	WaitForSemaphore();
	*aa = fItems[ReadIndex()].aa;
	*bb = fItems[ReadIndex()].bb;
	*cc = fItems[ReadIndex()].cc;
	*dd = fItems[ReadIndex()].dd;
	fReadPos++;
}

void ZzQueue::SignalSemaphore()
{
	semaphore_signal(fSemaphore);
}
	
void ZzQueue::WaitForSemaphore()
{
	semaphore_wait(fSemaphore);
}
	
